from time import sleep
import requests
import json
import random

class Search:
	def __init__(self, search_word, start_page=1, max_page=1):
		self.search_word = search_word
		self.start_page = start_page
		self.max_page = max_page

	def lastocc(snt, lst):
		rvs_lst = lst[::-1]
		snt_id = len(lst) - rvs_lst.index(snt) - 1

		return snt_id


	def get_result(self, url, page_num, max_page_num):
		'''
		Return: dict, page number: response of website
		Parameter: search_word -> string
		'''

		num = 1
		result = {}
		self.sleep = []
		while page_num <= max_page_num:
			search_url = url.format(self.search_word, page_num)
			page = requests.get(search_url)
			result.update({num: page.content})
			page_num += 10
			num += 1

			time_wait = random.uniform(1, 2)
			self.sleep.append(time_wait)
			sleep(time_wait)

		return result


	def res_to_dict(self, label, *data):
		'''
		Return: dict, all attribute in results.
		Parameter: 
		label -> list of string to be label of data.
		data -> list of row data and all attributes needed.
		'''

		self.dict_of_td = []
		for i, (d) in enumerate(zip(*data)):
			row = {'result_id': i}
			row.update({l:d[j] for j, l in enumerate(label)})
			self.dict_of_td.append(row)

		return self.dict_of_td


	def res_to_json(self):
		'''
		Return: json, all attribute in results.
		'''

		self.json_of_td = json.dumps(self.get_all(), indent=2)

		return self.json_of_td




from ..search import Search
from .google_search import GSearch
from .settings import GOOGLE_URLS

from bs4 import BeautifulSoup

class GBooks(GSearch):
	URL = GOOGLE_URLS['GOOGLE_BOOKS']
	LABELS = ['title', 'description', 'domain', 'link']

	def __init__(self, search_word, start_page=1, max_page=1):
		super().__init__(search_word, start_page, max_page)
		self.start_page_num = (start_page-1) * 10
		self.max_page_num = (max_page-1) * 10
		self.pages = self.get_result(GBooks.URL, self.start_page_num, self.max_page_num)

		self.titles = []
		self.descs = []
		self.domains = []
		self.links = []


	def get_desc(self):
		'''
		Return: description -> list.
		Example:
		>>> search_book = GBooks('Secret thing to get focus')
		>>> print(search_book.get_desc())
		'''

		if self.descs:
			return self.descs

		descs_tmp = self.get_main('div', {'class': lambda e: len(str(e).split()) == 3}, 
								  findAll=True)

		self.descs.extend([res[3].text if len(res) >= 4 else None for res in descs_tmp])
		return self.descs


	def get_all(self):
		'''
		Return: all attributes -> dict.
		Example:
		>>> search_book = GBooks('Secret thing to get focus')
		>>> print(search_book.get_all())
		'''

		self.get_title()
		self.get_desc()
		self.get_domainNlink('books.google.com')

		return self.res_to_dict(GBooks.LABELS, self.titles, self.descs, 
								self.domains, self.links)



from .google_search import GSearch
from .settings import GOOGLE_URLS

from bs4 import BeautifulSoup
import re

class GVideo(GSearch):
	'''
	GVideo is a class to gather result from Google Video Search (Not Youtube).
	GSearch --> Inherited.
	'''

	URL = GOOGLE_URLS['GOOGLE_VIDEO']
	LABELS = ['date', 'title', 'description', 'duration', 'domain', 'link']

	def __init__(self, search_word, start_page=1, max_page=1):
		super().__init__(search_word, start_page, max_page)
		self.start_page_num = (start_page-1) * 10
		self.max_page_num = (max_page-1) * 10
		self.pages = self.get_result(GVideo.URL, self.start_page_num, self.max_page_num)

		self.descs = []
		self.durs = []


	def get_desc(self):
		'''
		Return: description -> list, description of video.
		Example:
		>>> search_film = GSearch('Trailer: Godzilla vs Kong')
		>>> print(search_film.get_desc())
		'''

		if self.descs:
			return self.descs

		descs_tmp = self.get_main('div', {'class': lambda e: len(str(e).split()) == 3}, 
								  findAll=True)

		self.descs.extend([res[3].text if len(res) >= 4 else None for res in descs_tmp])
		return self.descs


	def get_dur(self):
		'''
		Return: duration -> list, duration of video.
		Example:
		>>> search_film = GSearch('Trailer: Godzilla vs Kong')
		>>> print(search_film.get_dur())
		'''

		if self.durs:
			return self.durs

		for df in self.descs:
			found = re.search(r"([0-9]+)?(:)?([0-5]?[0-9]):([0-5]?[0-9])", df)
			if found:
				self.durs.append(found.group(0))
			else:
				self.durs.append(None)

		return self.durs


	def run_some_clean(self):
		'''
		Return: None
		Result: Several variable value got cleaned data.
		'''

		# Remove duration and date from description
		split_sym = '·'
		junk_sym = '...'
		descs_tmp = []
		for d in self.descs:
			if junk_sym in d:
				d = d[:d.index(junk_sym)]

			if split_sym in d:
				d = d.split(split_sym)[1]

			descs_tmp.append(d)

		self.descs = descs_tmp


	def get_all(self):
		'''
		Return: all attributes of result -> dict.
		Example:
		>>> search_film = GSearch('Trailer: Godzilla vs Kong')
		>>> print(search_film.get_all())
		'''

		self.get_title()
		self.get_desc()
		self.get_date()
		self.get_dur()
		self.get_domainNlink('url?q=', '?')

		self.run_some_clean()

		return self.res_to_dict(GVideo.LABELS, self.dates, self.titles, 
								self.descs, self.durs, self.domains, self.links)



from ..search import Search
from .settings import GOOGLE_URLS

from bs4 import BeautifulSoup

class GShop(Search):
	'''
	GShop is a class for gather all top product from Google Shop Search.
	Search --> Inherited
	'''

	URL = GOOGLE_URLS['GOOGLE_SHOP']
	LABELS = ['title', 'price', 'e-commerce', 'src-link', 'img-link']

	def __init__(self, search_word, start_page=1, max_page=1):
		super().__init__(search_word, start_page, max_page)
		self.start_page_num = start_page * 10
		self.max_page_num = max_page * 10
		self.pages = self.get_result(GShop.URL, self.start_page_num, self.max_page_num)

		self.titles = []
		self.src_links = []
		self.img_links = []
		self.e_commerces = []
		self.prices = []


	def get_main(self, tag, attr, partial=False):
		'''
		Return: result -> list, the value is BS4 object. This function 
				is used by all another function inside this class.
		Parameter:
		tag -> string, html tag to gather.
		attr -> string, attribute from a tag. 
		[partial] !optional -> boolean, BS4 will match attr parameter 
							   with partial string, not full if True.
		Example:
		>>> search_assistant = GSearch('google assistant home')
		>>> print(search_assistant.get_main('a')) # you got <a> tag
		'''

		texts = []
		for page_num in self.pages:
			page = self.pages[page_num]
			soup = BeautifulSoup(page, 'html.parser')
			texts_parse = soup.findAll(tag, attr, partial=partial)

			attr_need = {'a': 'href', 'img': 'src'}
			if tag in attr_need:
				attr_specific = attr_need[tag]
				texts_ext = [t[attr_specific] for t in texts_parse]
			else:
				texts_ext = [t.text for t in texts_parse]

			texts.extend(texts_ext)

		return texts


	def get_title(self):
		'''
		Return: title -> list, title of product.
		Example: 
		>>> search_assistant = GSearch('google assistant home')
		>>> print(search_assistant.get_title())
		'''

		if self.titles:
			return self.titles

		self.titles.extend(self.get_main('div', {'class': 'sh-np__product-title translate-content'}))
		return self.titles


	def get_price(self):
		'''
		Return: price -> list, price of product.
		Example:
		>>> search_assistant = GSearch('google assistant home')
		>>> print(search_assistant.get_price())
		'''

		if self.prices:
			return self.prices

		price_tmp = self.get_main('b', None)
		self.prices.extend([p for p in price_tmp if 'Rp' in p])
		return self.prices


	def get_ecommerce(self):
		'''
		Return: e-commerce -> list, name of e-commerce platform to sell and buy product.
		Example:
		>>> search_assistant = GSearch('google assistant home')
		>>> print(search_assistant.get_ecommerce())
		'''

		if self.e_commerces:
			return self.e_commerces

		self.e_commerces.extend(self.get_main('div', {'class': 'sh-np__seller-container'}))
		return self.e_commerces


	def get_src_link(self):
		'''
		Return: source link -> list, link of product.
		Example:
		>>> search_assistant = GSearch('google assistant home')
		>>> print(search_assistant.get_src_link())
		'''

		if self.src_links:
			return self.src_links

		self.src_links.extend(self.get_main('a', {'class': 'sh-np__click-target'}))
		return self.src_links


	def get_img_link(self):
		'''
		Return: images link -> list, link of product image.
		Example:
		>>> search_assistant = GSearch('google assistant home')
		>>> print(search_assistant.get_img_link())
		'''

		if self.img_links:
			return self.img_links

		self.img_links.extend(self.get_main('img', {'role': 'presentation'}))
		return self.img_links


	def get_all(self):
		'''
		Return: all attributes of result -> dict.
		Example:
		>>> search_assistant = GSearch('google assistant home')
		>>> print(search_assistant.get_all())
		'''

		self.get_title()
		self.get_price()
		self.get_ecommerce()
		self.get_src_link()
		self.get_img_link()

		return self.res_to_dict(GShop.LABELS, self.titles, self.prices, 
								self.e_commerces, self.src_links, self.img_links)



from ..search import Search
from .settings import GOOGLE_URLS

from bs4 import BeautifulSoup
import re

class GScholar(Search):
	'''
	GScholar is a class to get result from Google Scholar Search.
	Search --> Inherited.
	'''

	URL = GOOGLE_URLS['GOOGLE_SCHOLAR']
	LABELS = ['year', 'title', 'description', 'authors', 'link', 
			  'pdf_link', 'journal_domain', 'domain', 'many_version']

	def __init__(self, search_word, start_page=1, max_page=1):
		super().__init__(search_word, start_page, max_page)
		self.start_page_num = (start_page-1) * 10
		self.max_page_num = (max_page-1) * 10
		self.pages = self.get_result(GScholar.URL, self.start_page_num, self.max_page_num)

		self.years = []
		self.titles = []
		self.descs = []
		self.authors = []
		self.links = []
		self.pdf_links = []
		self.journal_domains = []
		self.domains = []
		self.many_versions = []


	def get_main(self, tag, attr, findAll=False):
		'''
		Return: result -> list, the value is BS4 object. This function 
				is used by all another function inside this class.
		Parameter:
		tag -> string, html tag to gather.
		attr -> string, attribute from a tag. 
		[findAll] !optional -> boolean, the result will be 2D list instead of 1D, 
							   because BS4 will search all tag and attr 
							   that meet the criteria, not the first match.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_main('h3', {'class': 'gs_rt'})) # you get title
		'''

		res = []
		for page_num in self.pages:
			page = self.pages[page_num]
			soup = BeautifulSoup(page, 'html.parser')
			texts_parse = soup.findAll('div', {'class': 'gs_r gs_or gs_scl'})

			for t in texts_parse:
				get_attr = t.findAll(tag, attr) if findAll else t.find(tag, attr)

				if get_attr:
					res.append(get_attr)
				else:
					res.append(None)

		return res


	def get_year(self):
		'''
		Return: year -> list, year of result.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_year())
		'''

		if self.years:
			return self.years

		dates_tmp = self.get_main('div', {'class': 'gs_a'})
		for d in dates_tmp:
			d = d.text
			pattern = r"\d{4}"
			found_year = re.search(pattern, str(d))
			if found_year:
				self.years.append(found_year.group(0))
			else:
				self.years.append(None)

		return self.years


	def get_title(self):
		'''
		Return: title -> list, title of result.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_title())
		'''

		if self.titles:
			return self.titles

		res = [res.text for res in self.get_main('h3', {'class': 'gs_rt'})]
		self.titles.extend(res)
		return self.titles


	def get_desc(self):
		'''
		Return: description -> list, description of result.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_desc())
		'''

		if self.descs:
			return self.descs

		res = [res.text for res in self.get_main('div', {'class': 'gs_rs'})]
		self.descs.extend(res)
		return self.descs


	def get_author(self):
		'''
		Return: author -> list, author of result.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_author())
		'''

		if self.authors:
			return self.authors

		split_sym = '-'
		dates_tmp = self.get_main('div', {'class': 'gs_a'})
		for d in dates_tmp:
			d = d.text
			if split_sym in d:
				id_split_sym = d.index(split_sym)
				self.authors.append(d[:id_split_sym-1])
			else:
				self.authors.append(None)

		return self.authors


	def get_link(self):
		'''
		Return: link -> list, link of result.
		Example:
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_link())
		'''

		if self.links:
			return self.links

		res = [res.find('a')['href'] for res in self.get_main('h3', {'class': 'gs_rt'})]
		self.links.extend(res)

		return self.links


	def get_pdflink(self):
		'''
		Return: pdflink -> list, if the result have pdf, then the 
						   element must not be None.
		Example: 
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_pdflink())
		'''

		if self.pdf_links:
			return self.pdf_links

		self.pdf_links = [res.find('a')['href'] 
						 if res 
						 else None 
						 for res in self.get_main('div', {'class': 'gs_or_ggsm'})]

		return self.pdf_links


	def get_journal_domain(self):
		'''
		Return: journal_domain -> list, domain or subdomain that used 
								  for online publication.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_journal_domain())
		'''

		if self.journal_domains:
			return self.journal_domains

		split_sym = '-'
		journal_dom_tmp = self.get_main('div', {'class': 'gs_a'})
		for j in journal_dom_tmp:
			j = j.text
			pattern = r"([a-z0-9][a-z0-9\-]{0,61}[a-z0-9]\.)+[a-z0-9][a-z0-9\-]*[a-z0-9]"
			found_domain = re.search(pattern, str(j))
			if found_domain:
				self.journal_domains.append(found_domain.group(0))
			else:
				self.journal_domains.append(None)

		return self.journal_domains


	def get_domain(self):
		'''
		Return: domain -> list, domain or subdomain that extracted from link.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_domain())
		'''

		if self.domains:
			return self.domains

		subdomains = self.get_journal_domain()
		for s in subdomains:
			found_domain = re.search(r"(\w{2,}\.\w{2,3}\.\w{2,3}|\w{2,}\.\w{2,3})$", str(s))
			if found_domain:
				self.domains.append(found_domain.group(1))
			else:
				self.domains.append(None)

		return self.domains


	def get_many_version(self):
		'''
		Return: many of version -> list, available version of result.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_many_version())
		'''

		if self.many_versions:
			return self.many_versions

		self.many_versions = [res[-1].text
							  if res 
							  else None 
							  for res in self.get_main('a', {'class': 'gs_nph'}, findAll=True)]

		return self.many_versions


	def get_all(self):
		'''
		Return: all attributes of result -> dict.
		Example:
		>>> search_thesis = GScholar('informatics and technology thesis')
		>>> print(search_thesis.get_all())
		'''

		years = self.get_year()
		titles = self.get_title()
		descs = self.get_desc()
		authors = self.get_author()
		links = self.get_link()
		pdf_links = self.get_pdflink()
		journal_domains = self.get_journal_domain()
		domains = self.get_domain()
		many_versions = self.get_many_version()

		return self.res_to_dict(GScholar.LABELS, years, titles, descs, authors, 
								links, pdf_links, journal_domains, domains, many_versions)




GOOGLE_URLS = {'GOOGLE': 'https://www.google.com/search?q={}&start={}', 
			   'GOOGLE_SCHOLAR': 'https://scholar.google.com/scholar?q={}&start={}', 
			   'GOOGLE_VIDEO': 'https://www.google.com/search?q={}&tbm=vid&start={}', 
			   'GOOGLE_BOOKS': 'https://www.google.com/search?q={}&tbm=bks&start={}',
			   'GOOGLE_NEWS': 'https://www.google.com/search?q={}&tbm=nws&start={}', 
			   'GOOGLE_SHOP': 'https://www.google.com/search?q={}&tbm=shop&start={}'
			  }




#####################################################################################
# If you use this module as installed with pip, use import method as below.
#from Senginta.static.Google import GSearch, GNews, GBooks, GShop, GVideo, GScholar
#from Senginta.static.Baidu import BASearch
# ------------------------------------ OR
#from Senginta.static import Google, Baidu
# ------------------------------------
#####################################################################################


from static.Google import GSearch, GNews, GBooks, GShop, GVideo, GScholar
from static.Baidu import BASearch


# GoogleSearch = GSearch('Tokopedia')
# print(GoogleSearch.res_to_json())

# GoogleBooks = GBooks('Python Programming', 1, 3)
# print(GoogleBooks.res_to_json())

# GoogleNews = GNews('Idcloudhost', 1, 3) 
# print(GoogleNews.res_to_json())

# GoogleShops = GShop('Remote TV', 1, 3)
# print(GoogleShops.res_to_json())

# GoogleVideo = GVideo('Pegipegi', 1, 3)
# print(GoogleVideo.res_to_json())

# BaiduSearch = BASearch('Gojek', 1, 3)
# print(BaiduSearch.res_to_json())


###################################################################################
# If when you use Google Scrapper is not give anything, you must try
# manual search with format https://scholar.google.com/scholar?q=KEYWORD&start=0
# and pass the bot manually.
# Usually you got the additional parameter, paste that additional parameter 
# into below.
#GScholar.URL += "ANOTHER_PARAMETER_TO_PASS_BOT_PROTECTOR"
###################################################################################

# GoogleScholar = GScholar('Penggunaan Naive Bayes Classifier')
# print(GoogleScholar.res_to_json())


from ..search import Search
from .settings import BAIDU_URLS

from bs4 import BeautifulSoup
import re

class BASearch(Search):
	'''
	BASearch is a class to gather result from Baidu Search Engine.
	Search -> Inherited.
	'''

	URL = BAIDU_URLS['BAIDU_SEARCH']
	LABELS = ['title', 'description', 'author', 'author_link', 'domain', 'link']

	def __init__(self, search_word, start_page=1, max_page=1):
		super().__init__(search_word, start_page, max_page)
		self.start_page_num = (start_page-1) * 10
		self.max_page_num = (max_page-1) * 10
		self.pages = self.get_result(BASearch.URL, self.start_page_num, self.max_page_num)

		self.titles = []
		self.descs = []
		self.authors = []
		self.authors_link = []
		self.domains = []
		self.links = []


	def get_main(self, tag, attr=None, findAll=False):
		'''
		Return: result -> list, the value is BS4 object. This function 
				is used by all another function inside this class.
		Parameter:
		tag -> string, html tag to gather.
		[attr] !optional -> string, attribute from a tag. 
		[findAll] !optional -> boolean, the result will be 2D list instead of 1D, 
							   because BS4 will search all tag and attr 
							   that meet the criteria, not the first match.
		Example:
		>>> search_wschool = BASearch('when school')
		>>> print(search_wschool.get_main('a')) # you got <a> tag
		'''

		res = []
		for page_num in self.pages:
			page = self.pages[page_num]
			soup = BeautifulSoup(page, 'html.parser')
			texts_parse = soup.findAll('div', 
									   {'class': lambda e: e in ['result-op c-container new-pmd xpath-log', 
									   							 'result c-container new-pmd']
									   							 })

			for t in texts_parse:
				if attr:
					get_attr = t.findAll(tag, attr) if findAll else t.find(tag, attr)
				else:
					get_attr = t.findAll(tag) if findAll else t.find(tag)

				res.append(get_attr)

		return res


	def get_title(self):
		'''
		Return: title -> list, title of result.
		Example: 
		>>> search_wschool = BASearch('when school')
		>>> print(search_wschool.get_title())
		'''

		if self.titles:
			return self.titles

		res = [res.text 
			   if res 
			   else None 
			   for res in self.get_main('h3')]
		self.titles.extend(res)
		return self.titles


	def get_desc(self):
		'''
		Return: description -> list, description of result.
		Example:
		>>> search_wschool = BASearch('when school')
		>>> print(search_wschool.get_desc())
		'''

		if self.descs:
			return self.descs

		res = [res.text 
			   if res 
			   else None 
			   for res in self.get_main('div', {'class': lambda e: 'c-abstract' in str(e)})]
		self.descs.extend(res)
		return self.descs


	def get_authorNlink(self):
		'''
		Return: 
		 author -> list, author of result.
		 author link -> list, link to see profile of author.
		Example:
		>>> search_wschool = BASearch('when school')
		>>> print(search_wschool.get_authorNlink())
		'''

		if self.authors and self.authors_link:
			return self.authors, self.authors_link

		for res in self.get_main('a', {'class': 'c-gray'}, findAll=True):
			authors_tmp = []
			authors_link_tmp = []

			for author in res:
				if author:
					authors_tmp.append(author.text)
					authors_link_tmp.append(author['href'])
				else:
					authors_tmp.append(None)
					authors_link_tmp.append(None)

			self.authors.append(authors_tmp)
			self.authors_link.append(authors_link_tmp)

		return self.authors, self.authors_link


	def get_domainNlink(self):
		'''
		Return:
		 domain -> list, domain of result link.
		 link -> list, link of result.
		Example:
		>>> search_wschool = BASearch('when school')
		>>> print(search_wschool.get_domainNlink())
		'''

		if self.domains and self.links:
			return self.domains, self.links

		for res in self.get_main('a'):
			if res:
				link = res['href']
				domain = re.search(r"(?:^https?:\/\/([^\/]+)(?:[\/,]|$)|^(.*)$)", link)

				self.domains.append(domain.group(1))
				self.links.append(link)
			else:
				self.domains.append(None)
				self.links.append(None)

		return self.domains, self.links


	def get_all(self):
		'''
		Return: all attributes of result -> dict.
		Example:
		>>> search_wschool = BASearch('when school')
		>>> print(search_wschool.get_all())
		'''
		
		titles = self.get_title()
		descs = self.get_desc()
		authors, authors_link = self.get_authorNlink()
		domains, links = self.get_domainNlink()

		return self.res_to_dict(BASearch.LABELS, titles, descs, 
								authors, authors_link, domains, links)



from ..search import Search
from .settings import GOOGLE_URLS

from bs4 import BeautifulSoup
import re

class GSearch(Search):
	'''
	GSearch is a class for gather result from Google Search Engine.
	Search --> Inherited.
	'''
	
	URL = GOOGLE_URLS['GOOGLE']
	LABELS = ['date', 'title', 'description', 'domain', 'link']

	def __init__(self, search_word, start_page=1, max_page=1):
		super().__init__(search_word, start_page, max_page)
		self.start_page_num = (start_page-1) * 10
		self.max_page_num = (max_page-1) * 10
		self.pages = self.get_result(GSearch.URL, self.start_page_num, self.max_page_num)

		self.titles = []
		self.descs = []
		self.dates = []
		self.domains = []
		self.links = []


	def get_main(self, tag, attr=None, findAll=False):
		'''
		Return: result -> list, the value is BS4 object. This function 
				is used by all another function inside this class.
		Parameter:
		tag -> string, html tag to gather.
		[attr] !optional -> string, attribute from a tag. 
		[findAll] !optional -> boolean, the result will be 2D list instead of 1D, 
							   because BS4 will search all tag and attr 
							   that meet the criteria, not the first match.
		Example:
		>>> search_spider = GSearch('study from home')
		>>> print(search_spider.get_main('a')) # you got <a> tag
		'''

		res = []
		for page_num in self.pages:
			page = self.pages[page_num]
			soup = BeautifulSoup(page, 'html.parser')
			texts_parse = soup.findAll('div', {'class': lambda e: len(str(e).split()) == 4})

			for t in texts_parse:
				if attr:
					get_attr = t.findAll(tag, attr) if findAll else t.find(tag, attr)
				else:
					get_attr = t.findAll(tag) if findAll else t.find(tag)

				res.append(get_attr)

		return res


	def get_title(self):
		'''
		Return: title -> list, title of all results.
		Example: 
		>>> search_spider = GSearch('study from home')
		>>> print(search_spider.get_title())
		'''

		if self.titles:
			return self.titles

		titles_tmp = self.get_main('a')

		for res in titles_tmp:
			if res:
				title = res.find('h3')
				if title:
					self.titles.append(title.text)
					continue
			self.titles.append(None)

		return self.titles


	def get_desc(self):
		'''
		Return: description -> list, description of all results.
		Example: 
		>>> search_spider = GSearch('study from home')
		>>> print(search_spider.get_title())
		'''

		if self.descs:
			return self.descs

		descs_tmp = self.get_main('div', {'class': lambda e: len(str(e).split()) == 3}, 
								  findAll=True)

		self.descs.extend([res[2].text if len(res) >= 3 else None for res in descs_tmp])
		return self.descs


	def get_date(self):
		'''
		Return: date -> list, date of result.
		Example: 
		>>> search_spider = GSearch('study from home')
		>>> print(search_spider.get_date())
		'''

		if self.dates:
			return self.dates

		split_sym = '·'
		for desc in self.get_desc():
			date = None
			if split_sym in str(desc):
				date = desc.split(split_sym)[0]

			self.dates.append(date)
		return self.dates


	def get_domainNlink(self, keylink, spec_keylink=''):
		'''
		Return: 
		 domain -> list, domain of all results.
		 link -> list, link of all results.
		Example: 
		>>> search_spider = GSearch('study from home')
		>>> print(search_spider.get_domainNlink())
		'''

		if self.domains and self.links:
			return self.domains, self.links

		dlinks_tmp = self.get_main('a')

		for attr in dlinks_tmp:
			if attr:
				link = attr['href']
				if keylink in link:
					keylink_id = link.index(spec_keylink)
					link = link[keylink_id+3:]
			else:
				self.links.append(None)
				self.domains.append(None)
				continue
				
			domain = re.search(r"(?:^https?:\/\/([^\/]+)(?:[\/,]|$)|^(.*)$)", link)
			self.domains.append(domain.group(1))
			self.links.append(link)
			
		return self.domains, self.links


	def run_some_clean(self):
		'''
		Return: None
		Result: Several variable value got cleaned data.
		'''

		# Remove date from description
		split_sym = '·'
		self.descs = [d.split(split_sym)[1] if split_sym in str(d) else d for d in self.descs]


	def get_all(self):
		'''
		Return: 
		'''

		self.get_title()
		self.get_desc()
		self.get_date()
		self.get_domainNlink('url?q=', '?')

		self.run_some_clean()

		return self.res_to_dict(GSearch.LABELS, self.dates, self.titles, 
								self.descs, self.domains, self.links)
		


import setuptools

LIST_PACKAGES = ['senginta', 'senginta.static', 'senginta.static.Baidu', 'senginta.static.Google']

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="senginta",                        # This is the name of the package
    version="0.0.1",                        # The initial release version
    author="Michael Abraham Chan Tulenan",  # Full name of the author
    author_email="michael.4ct@gmail.com", 
    description="All in one Search Engine for used by API or Python Module (Unofficial)",
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    packages=LIST_PACKAGES,                 # List of all python modules to be installed
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.8',                # Minimum version requirement of the package
    py_modules=["senginta"],                # Name of the python package
    install_requires=['beautifulsoup4']     # Install other dependencies if any
)